package com.jiuqi.mde;

import java.util.Arrays;
import com.jiuqi.util.Guid;

public class MDERecord
{
    public Object refObj;
    protected MDEDataInfo dataInfo;
    protected MDEFieldSet fields;
    protected MDEDimValueSet dimValues;
    protected Object[] fieldValues;
    protected int recordFlag;
    public static final int RF_UNKNOWN = 0;
    public static final int RF_LOADED = 1;
    public static final int RF_MODIFIED = 2;
    public static final int RF_INSERTING = 4;
    public static final int RF_DELETING = 8;
    public static final int RF_DELETED = 16;
    
    public MDERecord(final MDEDataInfo dataInfo) {
        this.dataInfo = dataInfo;
        this.fields = dataInfo.fields();
        this.fieldValues = new Object[dataInfo.fields().size()];
    }
    
    public final MDEDimensionSet dimensions() {
        return this.dataInfo.dimensions();
    }
    
    public final MDEFieldSet fields() {
        return this.dataInfo.fields();
    }
    
    public final MDEDimValueSet getDimValues() {
        if (this.dimValues == null) {
            this.refreshDimValues(null);
        }
        return this.dimValues;
    }
    
    public void setDimValues(final MDEDimValueSet dimValues, final MDEIndex dvIndex) {
        if (dvIndex != null && this.dimValues != null) {
            dvIndex.remove((Object)this.dimValues);
        }
        this.getDimValues().assignValues(dimValues);
        for (int i = 0; i < this.dimensions().size(); ++i) {
            final MDEDimension dimension = this.dimensions().get(i);
            final MDEField field = this.fields.findByFieldName(dimension.fieldName);
            if (field != null) {
                this.setFieldValue(field, dimValues.getValue(dimension));
            }
        }
        if (dvIndex != null) {
            dvIndex.addlast((Object)this.dimValues, (Object)this);
        }
    }
    
    public void refreshDimValues(final MDEIndex dvIndex) {
        if (this.dimValues != null && !this.isModified()) {
            return;
        }
        if (dvIndex != null && this.dimValues != null) {
            dvIndex.remove((Object)this.dimValues);
        }
        if (this.dimValues == null) {
            this.dimValues = new MDEDimValueSet(this.dataInfo.dimensions());
        }
        for (int i = 0; i < this.dimensions().size(); ++i) {
            final MDEDimension dimension = this.dimensions().get(i);
            final MDEField field = this.fields.findByFieldName(dimension.fieldName);
            if (field != null) {
                this.dimValues.setValue(dimension, this.getFieldValue(field));
            }
        }
        if (dvIndex != null) {
            dvIndex.addlast((Object)this.dimValues, (Object)this);
        }
    }
    
    protected final Object getFieldValue(final int index) {
        return this.fieldValues[index];
    }
    
    protected final Object getFieldValue(final MDEField field) {
        try {
            final int index = this.fields.indexOf(field);
            return this.fieldValues[index];
        }
        catch (final ArrayIndexOutOfBoundsException ex) {
            throw new ArrayIndexOutOfBoundsException("找不到字段：" + field.fieldName);
        }
    }
    
    protected final void setFieldValue(final int index, final Object value) {
        if ((value == null && this.fieldValues[index] != null) || (value != null && !value.equals(this.fieldValues[index]))) {
            this.recordFlag |= 0x2;
        }
        this.fieldValues[index] = value;
    }
    
    protected final void setFieldValue(final MDEField field, final Object value) {
        final int index = this.fields.indexOf(field);
        if (index < 0) {
            throw new ArrayIndexOutOfBoundsException("找不到字段：" + field.fieldName);
        }
        if ((value == null && this.fieldValues[index] != null) || (value != null && !value.equals(this.fieldValues[index]))) {
            this.recordFlag |= 0x2;
        }
        this.fieldValues[index] = value;
    }
    
    public final void copyFieldValue(final MDERecord source, final int index) {
        final Object value = source.fieldValues[index];
        if ((value == null && this.fieldValues[index] != null) || (value != null && !value.equals(this.fieldValues[index]))) {
            this.recordFlag |= 0x2;
        }
        this.fieldValues[index] = value;
    }
    
    public final void generateIdenty() {
        this.setFieldValue(this.dataInfo.getIndentyField(), Guid.newGuid());
    }
    
    public boolean hasData() {
        for (int i = 0; i < this.fieldValues.length; ++i) {
            final MDEField field = this.fields.get(i);
            if (field.object != null && this.fieldValues[i] != null) {
                return true;
            }
        }
        return false;
    }
    
    public final boolean isInserted() {
        return (this.recordFlag & 0x4) != 0x0;
    }
    
    public final boolean isModified() {
        return (this.recordFlag & 0x2) != 0x0;
    }
    
    public final boolean isDeleting() {
        return (this.recordFlag & 0x8) != 0x0;
    }
    
    public final boolean isDeleted() {
        return (this.recordFlag & 0x10) != 0x0;
    }
    
    public final void setIsInserted() {
        this.recordFlag = 6;
    }
    
    public final void clearRecordFlag() {
        this.recordFlag = 0;
    }
    
    public final String getRecordState() {
        if (this.isDeleted()) {
            return "Deleted";
        }
        if (this.isDeleting()) {
            return "Deleting";
        }
        if (this.isInserted()) {
            return "Inserted";
        }
        if (this.isModified()) {
            return "Modified";
        }
        return "Default";
    }
    
    protected void beforeNewRecord() throws MDEException {
    }
    
    protected void afterNewRecord() throws MDEException {
        this.recordFlag = 4;
    }
    
    protected void beforeLoadRecord() throws MDEException {
    }
    
    protected void afterLoadRecord() throws MDEException {
        this.recordFlag = 1;
    }
    
    public void checkModification() throws MDEException {
    }
    
    protected void beforeSaveRecord() throws MDEException {
    }
    
    protected void afterSaveRecord() throws MDEException {
        this.recordFlag = 1;
    }
    
    protected void beforeDeleteRecord() throws MDEException {
    }
    
    protected void afterDeleteRecord() throws MDEException {
        this.recordFlag = 16;
    }
    
    protected void calculateDefaultValues() throws MDEException {
    }
    
    public void delete() {
        this.recordFlag |= 0x8;
    }
    
    public final void emptyRecord() {
        this.recordFlag = 0;
        Arrays.fill(this.fieldValues, null);
    }
    
    public final void assignValues(final MDERecord source) {
        for (int i = 0; i < this.fieldValues.length; ++i) {
            this.fieldValues[i] = source.fieldValues[i];
        }
        this.recordFlag |= 0x2;
    }
    
    public final void negateValues() {
        for (int i = 0; i < this.fieldValues.length; ++i) {
            final Object value = this.fieldValues[i];
            if (value instanceof Integer) {
                this.fieldValues[i] = new Integer(-(int)this.fieldValues[i]);
            }
            else if (value instanceof Double) {
                this.fieldValues[i] = new Double(-(double)this.fieldValues[i]);
            }
        }
        this.recordFlag |= 0x2;
    }
}
